/**
 * @module dixim
 * @submodule vdms
 */
dixim.namespace('dixim.vdms');

/**
 * @module contents
 * @submodule progress-dialog-controller
 */

(function() {
var ProgressDialogController;
var prototype;
var superConstructor;
var ProgressDialogView = dixim.vdms.ProgressDialogView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var ApiClientVdms = dixim.common.ApiClientVdms;
var Enum = dixim.ctt.util.Enum;

/**
 * @class ProgressDialogController
 * @constructor
 * @param {Object} configs
 */
ProgressDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ProgressDialogController, EventTarget);
dixim.vdms.ProgressDialogController = ProgressDialogController;
prototype = ProgressDialogController.prototype;
superConstructor = ProgressDialogController.superConstructor;

var YES = 'yes';
var COMPLETE = 'complete';
var CLOSE = 'close';
var UPDATE = 'update';

var Mode = Enum.fetch('PROGRESS');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'progress-dialog'].join('');

    this._mode = configs.mode;

    this._model = new dixim.common.ApiClientVdms();
    switch(this._mode) {
        case Mode.NON_AGGREGATION:
            this._model.addEventListener('disableAggregation', this._success, this);
            this._model.addEventListener('disableAggregationError', this._error, this);
            break;
        case Mode.AGGREGATION:
            this._model.addEventListener('enableAggregation', this._success, this);
            this._model.addEventListener('enableAggregationError', this._error, this);
            break;
        case Mode.PAUSE:
            this._model.addEventListener('cancelSyncAggregation', this._success, this);
            this._model.addEventListener('cancelSyncAggregationError', this._error, this);
            break;
        case Mode.RESTART:
            this._model.addEventListener('syncAggregation', this._success, this);
            this._model.addEventListener('syncAggregationError', this._error, this);
            break;
    }

    this._view = new ProgressDialogView(configs);
    this._view.addEventListener(YES, this._clickYes, this);
    this._view.addEventListener(COMPLETE, this.complete, this);

    this._contents = configs.contents;
    this._successContents = [];
    this._errorContents = [];
    this._isCancel = false;
    this._sessions = [];
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._success = function(e) {
    var contentsLen = this._contents.length;

    for (var i = 0; i < contentsLen; i += 1) {
        if (this._contents[i].udn === e.args.session.request.udn) {
            this._successContents.push($.extend(true, {}, this._contents[i]))
        }
    }
    this._view.decrementDescription();
};

prototype._error = function(e) {
    var contentsLen = this._contents.length;

    for (var i = 0; i < contentsLen; i += 1) {
        if (this._contents[i].udn === e.args.session.request.udn) {
            this._errorContents.push($.extend(true, {errorCode: e.args.data.errorCode || 0}, this._contents[i]))
        }
    }

    this._view.decrementDescription();
};

prototype._executeContents = function() {
    var contentsLen = this._contents.length;
    var query = {};

    this._view.setButtons([
        {
            text: _('dialog_cancel'),
            click: dixim.proxy(this._clickCancel, this), 
        },
    ]);

    this._isCancel = false;
    this._errorContents = [];
    for (var i = 0; i < contentsLen; i += 1) {
        switch(this._mode) {
            case Mode.NON_AGGREGATION:
                query.udn = this._contents[i].udn;
                this._sessions.push(this._model.disableAggregation(query));
                break;
            case Mode.AGGREGATION:
                query.udn = this._contents[i].udn;
                this._sessions.push(this._model.enableAggregation(query));
                break;
            case Mode.PAUSE:
                query.udn = this._contents[i].udn;
                this._sessions.push(this._model.cancelSyncAggregation(query));
                break;
            case Mode.RESTART:
                query.udn = this._contents[i].udn;
                this._sessions.push(this._model.syncAggregation(query));
                break;
        }
    }
};

prototype.complete = function() {
    var contentsLen = this._contents.length;
    var successContentsLen = this._successContents.length;
    var isSuccess;

    this.stop();

    for (var i = 0; i < contentsLen; i += 1) {
        isSuccess = false;
        for (var j = 0; j < successContentsLen; j += 1) {
            if (this._contents[i].udn === this._successContents[j].udn) {
                isSuccess = true;
                break;
            }
        }
        if (!isSuccess) {
            this._errorContents.push(true, $.extend({errorCode: 0}, this._contents[i]))
        }
    }

    if (this._errorContents.length !== 0) {
        switch(this._mode) {
            case Mode.NON_AGGREGATION:
                this._errorDialog(_('vdms_aggregation_operation_non_aggregate_error'));
                break;
            case Mode.AGGREGATION:
                this._errorDialog(_('vdms_aggregation_operation_aggregate_error'));
                break;
            case Mode.PAUSE:
                this._errorDialog(_('vdms_aggregation_operation_pause_error'));
                break;
            case Mode.RESTART:
                this._errorDialog(_('vdms_aggregation_operation_restart_error'));
                break;
        }
        this.dispatchEvent(new Event(CLOSE));
    } else {
        this.dispatchEvent(new Event(UPDATE));
    }
};

prototype._clickYes = function() {
    this._executeContents();
};

prototype._clickCancel = function() {
    this._isCancel = true;

    for (var i = 0; i < this._sessions.length; i += 1) {
        this._sessions[i].cancel();
    }
    this.complete();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;
    var errorContentsLen = this._errorContents.length;

    switch(this._mode) {
        case Mode.NON_AGGREGATION:
            query.title = _('vdms_aggregation_operation_non_aggregate');
            break;
        case Mode.AGGREGATION:
            query.title = _('vdms_aggregation_operation_aggregate');
            break;
        case Mode.PAUSE:
            query.title = _('vdms_aggregation_operation_pause');
            break;
        case Mode.RESTART:
            query.title = _('vdms_aggregation_operation_restart');
            break;
    }

    query.id = this._id;
    query.doubleDialog = true;
    query.contentHeight = '200px';
    query.description = (this._isCancel) ? _('vdms_aggregation_cancel') : text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();

    for (var i = 0; i < errorContentsLen; i += 1) {
        dialog.addErrorList(this._errorContents[i].friendlyName, _(['vdms_aggregation_error_', this._errorContents[i].errorCode].join('')));
    }
    dialog.removeClass('error-dialog-body-content');
    dialog.removeClass('error-text-field');
};

}());
