/**
 * @module ctt
 * @submodule locking-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var LockingView = dixim.ctt.LockingView;
var AutoDeleteApiClient = dixim.ctt.api.AutoDeleteApiClient;
var _ = dixim.ctt.ResourceBundle.get;

var LockingController;
var prototype;

/**
 * @class LockingController
 * @constructor
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
LockingController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(LockingController, EventTarget);
dixim.ctt.LockingController = LockingController;
prototype = LockingController.prototype;

/**
 * Triggered when the locking has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.locked true if the locking is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = configs.model || new AutoDeleteApiClient();
  this._targetItems = configs.targetItems;
  this._view = new LockingView({targetItems: this._targetItems});
  this._session = null;
  this._queue = [];
  this._locked = false;
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var m = this._model;

  v.addEventListener('lock', this._didClickLock, this);
  v.addEventListener('cancel', this._didClickCancel, this);

  m.addEventListener('lock', this._didActionLock, this);
  m.addEventListener('lockError', this._didFailActionLock, this);

  v.show();
};

prototype._didClickLock = function(e) {
  if (this._queue.length) return;

  this._view.showActivityIndicator();

  this._queue = this._targetItems.concat();
  this._locked = true;
  this._processQueue();
};

prototype._processQueue = function() {
  var item;
  var total = this._targetItems.length;
  var queue = this._queue;

  if (!queue.length) {
    this._didFinishLocking();
    return;
  }

  item = queue.shift();
  this._view.updateProgress(item, total - queue.length - 1, total);
  this._session = this._model.lock({objectIds: item.id, protectFlag: true});
};

prototype._didActionLock = function(e) {
  this._session = null;
  this._processQueue();
};

prototype._didFailActionLock = function(e) {
  this._view.hideActivityIndicator();
  this._session = null;
  this._queue = [];
  this._view.showError(_('ctt_locking_view_error'));
};

prototype._didClickCancel = function(e) {
  if (!this._queue.length && !!!this._session) {
    this._view.destroy();
    this.dispatchEvent(new Event(CLOSE, {locked: this._locked}));
    return;
  }
  this._queue = [];
};

prototype._didFinishLocking = function(e) {
  this._view.hideActivityIndicator();
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE, {locked: this._locked}));
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

}());
