/**
 * @module ctt
 * @submodule ctt-api
 */
dixim.namespace('dixim.ctt.api');

/**
 * @module ctt-api
 * @submodule contents-transfer-api-client
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var Formatter = dixim.ctt.util.Formatter;
var ContentsTransferApiClientSession = dixim.ctt.api.ContentsTransferApiClientSession;
var Configs = dixim.ctt.Configs;
var $ = dixim.ctt.jQuery;

var ContentsTransferApiClient;
var prototype;

/**
 * Provides interfaces to Contents Transfer API.
 *
 * @class ContentsTransferApiClient
 * @param {Object} [configs]
 */
ContentsTransferApiClient = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ContentsTransferApiClient, EventTarget);
dixim.ctt.api.ContentsTransferApiClient = ContentsTransferApiClient;
prototype = ContentsTransferApiClient.prototype;

/**
 * Triggered when the browse action was succeeded.
 *
 * @event browse
 */
var BROWSE = 'browse';

/**
 * Triggered when the browse action was failed.
 *
 * @event browseError
 */
var BROWSE_ERROR = 'browseError';

/**
 * Triggered when the search action was succeeded.
 *
 * @event search
 */
var SEARCH = 'search';

/**
 * Triggered when the search action was failed.
 *
 * @event searchError
 */
var SEARCH_ERROR = 'searchError';

/**
 * Triggered when the storage_list action was succeeded.
 *
 * @event storageList
 */
var STORAGE_LIST = 'storageList';

/**
 * Triggered when the storage_list action was failed.
 *
 * @event storageListError
 */
var STORAGE_LIST_ERROR = 'storageListError';

/**
 * Triggered when the dms_list action was succeeded.
 *
 * See API docs for detail information of args.data.
 *
 * @event dmsList
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.data
 * @param {Object[]} event.args.data.results
 */
var DMS_LIST = 'dmsList';

/**
 * Triggered when the dms_list action was failed.
 *
 * @event dmsListError
 */
var DMS_LIST_ERROR = 'dmsListError';

/**
 * Triggered when the dn_dms_list action was succeeded.
 *
 * See API docs for detail information of args.data.
 *
 * @event dmsList
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.data
 * @param {Object[]} event.args.data.results
 */
var DN_DMS_LIST = 'dnDmsList';

/**
 * Triggered when the dn_dms_list action was failed.
 *
 * @event dmsListError
 */
var DN_DMS_LIST_ERROR = 'dnDmsListError';

/**
 * Triggered when the remove_dms action was succeeded.
 *
 * @event removeDms
 */
var REMOVE_DMS = 'removeDms';

/**
 * Triggered when the remove_dms action was failed.
 *
 * @event removeDmsError
 */
var REMOVE_DMS_ERROR = 'removeDmsError';

/**
 * Triggered when the rec_dest_list action was succeeded.
 *
 * @event recDestList
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.data
 * @param {Object[]} event.args.data.results
 */
var REC_DEST_LIST = 'recDestList';

/**
 * Triggered when the rec_dest_list action was failed.
 *
 * @event recDestListError
 */
var REC_DEST_LIST_ERROR = 'recDestListError';

/**
 * Triggered when the transfer action was succeeded.
 *
 * @event transfer
 */
var TRANSFER = 'transfer';

/**
 * Triggered when the transfer action was failed.
 *
 * @event tarnsferError
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.errorCode;
 */
var TRANSFER_ERROR = 'transferError';

/**
 * Triggered when the dn_transfer action was succeeded.
 *
 * @event dnTranser
 */
var DN_TRANSFER = 'dnTransfer';

/**
 * Triggered when the dn_transfer action was failed.
 *
 * @event dnBrowseError
 */
var DN_TRANSFER_ERROR = 'dnTransferError';

/**
 * Triggered when the transfer_status action was succeeded.
 *
 * @event transferStatus
 */
var TRANSFER_STATUS = 'transferStatus';

/**
 * Triggered when the transfer_status action was failed.
 *
 * @event transferStatusError
 */
var TRANSFER_STATUS_ERROR = 'transferStatusError';

/**
 * Triggered when the cancel_transfer action was succeeded.
 *
 * @event cancelTransfer
 */
var CANCEL_TRANSFER = 'cancelTransfer';

/**
 * Triggered when the cancel_transfer action was failed.
 *
 * @event cancelTransferError
 */
var CANCEL_TRANSFER_ERROR = 'cancelTransferError';

/**
 * Triggered when the rename action was succeeded.
 *
 * @event rename
 */
var RENAME = 'rename';

/**
 * Triggered when the rename action was failed.
 *
 * @event renameError
 */
var RENAME_ERROR = 'renameError';

/**
 * Triggered when the get token action was succeeded.
 *
 * @event getToken
 */
var GET_TOKEN = 'getToken';

/**
 * Triggered when the get token action was failed.
 *
 * @event getTokenError
 */
var GET_TOKEN_ERROR = 'getTokenError';

/**
 * Triggered when the remove action was succeeded.
 *
 * @event remove
 */
var REMOVE = 'remove';

/**
 * Triggered when the remove action was failed.
 *
 * @event removeError
 */
var REMOVE_ERROR = 'removeError';

/**
 * Triggered when the match_pin action was succeeded.
 *
 * @event matchPinCode
 */
var MATCH_PIN_CODE = "matchPinCode";

/**
 * Triggered when the match_pin action was failed.
 *
 * @event matchPinCodeError
 */
var MATCH_PIN_CODE_ERROR = "matchPinCodeError";

/**
 * Triggered when the log_list action was succeeded.
 *
 * @event logList
 */
var LOG_LIST = "logList";

/**
 * Triggered when the log_list action was failed.
 *
 * @event logListError
 */
var LOG_LIST_ERROR = "logListError";

/**
 * Triggered when the clear_log_list action was succeeded.
 *
 * @event clearLogList
 */
var CLEAR_LOG_LIST = "clearLogList";

/**
 * Triggered when the clear_log_list action was failed.
 *
 * @event clearLogListError
 */
var CLEAR_LOG_LIST_ERROR = "clearLogListError";

/**
 * Triggered when the reservation_list action was succeeded.
 *
 * @event reservationList
 */
var RESERVATION_LIST = "reservationList";

/**
 * Triggered when the reservation_list action was failed.
 *
 * @event reservationListError
 */
var RESERVATION_LIST_ERROR = "reservationListError";

/**
 * Triggered when the update_reservation action was succeeded.
 *
 * @event updateReservation
 */
var UPDATE_RESERVATION = 'updateReservation';

/**
 * Triggered when the update_reservation action was failed.
 *
 * @event updateReservationError
 */
var UPDATE_RESERVATION_ERROR = 'updateReservationError';

/**
 * Triggered when the remove_reservation action was succeeded.
 *
 * @event removeReservation
 */
var REMOVE_RESERVATION = 'removeReservation';

/**
 * Triggered when the remove_reservation action was failed.
 *
 * @event removeReservationError
 */
var REMOVE_RESERVATION_ERROR = 'removeReservationError';

/**
 * Triggered when the get_requested_transfer_list action was succeeded.
 *
 * @event getRequestedTransferList
 */
var GET_REQUESTED_TRANSFER_LIST = 'getRequestedTransferList';

/**
 * Triggered when the get_requested_transfer_list action was failed.
 *
 * @event getRequestedTransferListError
 */
var GET_REQUESTED_TRANSFER_LIST_ERROR = 'getRequestedTransferListError';

/**
 * Triggered when the dn_browse action was succeeded.
 *
 * @event dnBrowse
 */
var DN_BROWSE = 'dnBrowse';

/**
 * Triggered when the dn_browse action was failed.
 *
 * @event dnBrowseError
 */
var DN_BROWSE_ERROR = 'dnBrowseError';

/**
 * Triggered when the fetch availability action was succeeded.
 *
 * @event fetchAvailabilityCompression
 */
var FETCH_AVAILABILITY_COMPRESSION = 'fetchAvailabilityCompression';

/**
 * Triggered when the fetch availability action was failed.
 *
 * @event fetchAvailabilityCompressionError
 */
var FETCH_AVAILABILITY_COMPRESSION_ERROR = 'fetchAvailabilityCompressionError';

/**
 * Triggered when the update availability action was succeeded.
 *
 * @event updateAvailabilityCompression
 */
var UPDATE_AVAILABILITY = 'updateAvailailityCompression';

/**
 * Triggered when the update availability action was failed.
 *
 * @event updateAvailabilityCompressionError
 */
var UPDATE_AVAILABILITY_ERROR = 'updateAvailabilityCompressionError';

/**
 * Triggered when the fetch preset action was succeeded.
 *
 * @event fetchPreset
 */
var FETCH_PRESET = 'fetchPreset';

/**
 * Triggered when the fetch preset action was failed.
 *
 * @event fetchPresetError
 */
var FETCH_PRESET_ERROR = 'fetchPresetError';

/**
 * Triggered when the update preset action was succeeded.
 *
 * @event updatePreset
 */
var UPDATE_PRESET = 'updatePreset';

/**
 * Triggered when the update preset action was failed.
 *
 * @event updatePresetError
 */
var UPDATE_PRESET_ERROR = 'updatePresetError';

/**
 * Triggered when the reservation compression action was succeeded.
 *
 * @event reservationCompression
 */
var RESERVATION_COMPRESSION = 'reservationCompression';

/**
 * Triggered when the reservation compression action was failed.
 *
 * @event reservationCompressionError
 */
var RESERVATION_COMPRESSION_ERROR = 'reservationCompressionError';

/**
 * Triggered when the pin code that is set to the API client is changed.
 *
 * @event changePin
 * @param {Event} event
 * @param {Object} event.args
 * @param {String} event.args.pin
 */
var CHANGE_PIN = "changePin";

/**
 * @method _initialize
 * @protected
 * @param {Object} [configs]
 * @param {String} [configs.pin]
 */
prototype._initialize = function(configs) {
  configs = configs || {};
  this._sessions = [];
  this._pin = typeof configs.pin === 'undefined' ? null : configs.pin;
};

/**
 * Sets PIN code.
 *
 * This method just sets the value to the instance
 * and triggers <em>changePin</em> event.
 *
 * (i.e. It's required to verify the PIN code by
 *       matchPinCode() prior to setting.)
 *
 * @method setPin
 * @param {String} pin
 */
prototype.setPin = function(pin) {
  this._setPin(pin);
};

prototype._setPin = function(pin) {
  if (this._pin === pin) return;
  this._pin = pin;
  this.dispatchEvent(new Event(CHANGE_PIN, {pin: pin}));
};

/**
 * @method _createQueryData
 * @private
 */
prototype._createQueryData = function(request) {
  var query = {};
  var criteria;
  var value;
  var pin;

  var getCdsPropertyName = function(key) {
    if (key === 'title') {
      return 'dc:title';
    } else if (key === 'date') {
      return 'dc:date';
    }
    return null;
  };

  var makeSortCriteria = function(kva) {
    var len = kva.length;
    var tmp = [];
    var i;
    var k;
    var o;

    for (i = 0; i < len; i += 1) {
      if (!!!(k = getCdsPropertyName(kva[i].key))) continue;
      o = kva[i].order ? '-' : '+';
      tmp.push(o + k);
    }
    return tmp.join(',');
  };

  if (typeof request.udn !== 'undefined') {
    query.udn = request.udn;
  }

  query.id = request.id;

  if (typeof request.startingIndex !== 'undefined') {
    query.starting_index = request.startingIndex;
  }

  if (typeof request.requestedCount !== 'undefined') {
    query.requested_count = request.requestedCount;
  }

  if (!!request.browseFlag) {
    query.browse_flag = request.browseFlag;
  }

  if (!!request.filter) {
    query.filter = request.filter;
  }

  if (!!request.sortCriteria) {
    query.sort_criteria = makeSortCriteria(request.sortCriteria);
  }

  if (!!(criteria = request.searchCriteria)) {
    if (!!(value = criteria['title'])) {
      query.key = 'title';
      query.value = value;
    }
  }

  pin = request.pin || this._pin;
  if (!!pin) {
    query.pin_code = pin;
  }

  if (request.rating && query.pin_code) {
    query.rating = request.rating;
  }

  return query;
};

/**
 * Browses the direct children of the container specified.
 *
 * @method browse
 * @param {Object} request
 * @param {String} request.id an object ID to be browsed.
 * @param {String} [request.browseFlag] 'children' or 'metadata'.
 * @param {Number} [request.startingIndex]
 * @param {Number} [request.requestedCount]
 * @param {String} [request.filter] 'item' or 'container'.
 * @param {Object[]} [request.sortCriteria] [{key: xxx, order: true|false}].
 * @param {String} [request.pin]
 * @return {ContentsTransferApiClientSession}
 */
prototype.browse = function(request) {
  var query;
  var session;
  var cfg;

  query = this._createQueryData(request);
  session = new ContentsTransferApiClientSession('browse');

  cfg = {
    type: 'POST',
    data: query,
    dataType: 'json',
    success: this._makeResponseHandler(this._didBrowse, [session]),
    error: this._makeResponseHandler(this._didFailBrowse, [session]),
    context: this
  };
  session.query = query;
  session.request = request;
  session.connection = $.ajax(Configs.get('api') + '/browse', cfg);
  this._sessions.push(session);

  return session;
};

prototype._didBrowse = function(data, textStatus, jqXhr, session) {
  var request = session.request;
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.parentId = request.id;
    args.data = data;

    event = new Event(BROWSE, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailBrowse = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, BROWSE_ERROR);
};

prototype._makeResponseHandler = function(handler, args) {
  var slice = Array.prototype.slice;
  return $.proxy(function() {
    handler.apply(this, slice.call(arguments, 0).concat(args));
  }, this);
};

prototype._handleFailureResponse = function(session, errorCode, eventType) {
  dixim.array.remove(this._sessions, session);
  this.dispatchEvent(new Event(eventType, {session: session, errorCode: errorCode}));
};

/**
 * Browses the container direct children of the remote server specified.
 *
 * @method dnBrowse
 * @param {Object} request
 * @param {String} request.udn an udn of the server to be browsed.
 * @param {String} request.id an object ID to be browsed.
 * @param {String} [request.browseFlag] 'children' or 'metadata'.
 * @param {Number} [request.startingIndex]
 * @param {Number} [request.requestedCount]
 * @param {String} [request.filter] 'item' or 'container'.
 * @param {Object[]} [request.sortCriteria] [{key: xxx, order: true|false}].
 * @param {String} [request.pin]
 * @param {Boolean} [request.rating]
 * @return {ContentsTransferApiClientSession}
 */
prototype.dnBrowse = function(request) {
  var query;
  var session;
  var cfg;
  var api = 'dn_browse';

  query = this._createQueryData(request);
  session = new ContentsTransferApiClientSession(api);

  cfg = {
    type: 'POST',
    data: query,
    dataType: 'json',
    success: this._makeResponseHandler(this._didDnBrowse, [session]),
    error: this._makeResponseHandler(this._didFailDnBrowse, [session]),
    context: this
  };
  session.query = query;
  session.request = request;
  session.connection = $.ajax([Configs.get('api'), '/', api].join(''), cfg);
  this._sessions.push(session);

  return session;
};

prototype._didDnBrowse = function(data, textStatus, jqXhr, session) {
  var request = session.request;
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.parentId = request.id;
    args.data = data;

    event = new Event(DN_BROWSE, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailDnBrowse = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, DN_BROWSE_ERROR);
};

/**
 * Searches the contents for the searchCriteria specified.
 *
 * @method search
 * @param {Object} request
 * @param {String} request.id a container object ID to be searched.
 * @param {Number} [request.startingIndex]
 * @param {Number} [request.requestedCount]
 * @param {Object[]} [request.sortCriteria] {key: xxx, order: true|false}
 * @param {Object} [request.searchCriteria] {key: value}
 * @param {String} [request.pin]
 * @return {ContentsTransferApiClientSession}
 */
prototype.search = function(request) {
  var query;
  var session;
  var cfg;

  query = this._createQueryData(request);
  session = new ContentsTransferApiClientSession('search');

  cfg = {
    type: 'POST',
    data: query,
    dataType: 'json',
    success: this._makeResponseHandler(this._didSearch, [session]),
    error: this._makeResponseHandler(this._didFailSearch, [session]),
    context: this
  };
  session.query = query;
  session.request = request;
  session.connection = $.ajax(Configs.get('api') + '/search', cfg);

  this._sessions.push(session);
  return session;
};

prototype._didSearch = function(data, textStatus, jqXhr, session) {
  var request = session.request;
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.parentId = request.id;
    args.data = data;

    event = new Event(SEARCH, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailSearch = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, SEARCH_ERROR);
};

/**
 * Fetches the storage list.
 *
 * @method storageList
 * @return {ContentsTransferApiClientSession}
 */
prototype.storageList = function() {
  var api = 'storage_list';
  var session;
  var req;

  session = new ContentsTransferApiClientSession(api);

  req = {
    cache: false,
    success: this._makeResponseHandler(this._didFetchStorageList, [session]),
    error: this._makeResponseHandler(this._didFailFetchStorageList, [session]),
    dataType: 'json',
    context: this
  };
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didFetchStorageList = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(STORAGE_LIST, args);
  this.dispatchEvent(e);
};

prototype._didFailFetchStorageList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, STORAGE_LIST_ERROR);
};

/**
 * Fetches the DMS list.
 *
 * @method dmsList
 * @return {ContentsTransferApiClientSession}
 */
prototype.dmsList = function() {
  return this._fetchDmsListImpl('dms_list', this._didFetchDmsList, this._didFailFetchDmsList);
};

prototype._didFetchDmsList = function(data, textStatus, jqXhr, session) {
  this._fetchDmsListSuccessHandlerImpl(DMS_LIST, data, textStatus, jqXhr, session);
};

prototype._didFailFetchDmsList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, DMS_LIST_ERROR);
};

/**
 * Fetches the DMS list which support DTCP-IP.
 *
 * @method dnDmsList
 * @return {ContentsTransferApiClientSession}
 */
prototype.dnDmsList = function() {
  return this._fetchDmsListImpl('dn_dms_list', this._didFetchDnDmsList, this._didFailFetchDnDmsList);
};

prototype._didFetchDnDmsList = function(data, textStatus, jqXhr, session) {
  this._fetchDmsListSuccessHandlerImpl(DN_DMS_LIST, data, textStatus, jqXhr, session);
};

prototype._didFailFetchDnDmsList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, DN_DMS_LIST_ERROR);
};

prototype._fetchDmsListImpl = function(api, successHanlder, failureHandler) {
  var session;
  var req;

  session = new ContentsTransferApiClientSession(api);

  req = {
    success: this._makeResponseHandler(successHanlder, [session]),
    error: this._makeResponseHandler(failureHandler, [session]),
    cache: false,
    dataType: 'json',
    context: this
  };
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._fetchDmsListSuccessHandlerImpl = function(eventType, data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(eventType, args);
  this.dispatchEvent(e);
};

/**
 * Removes a DMS from the dms_list/dn_dms_list result.
 *
 * @method removeDms
 * @param {Object} request
 * @param {String} request.udn
 * @return {ContentsTransferApiClientSession}
 */
prototype.removeDms = function(request) {
  var api = 'remove_dms';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  query.udn = request.udn;

  req = {
    cache: false,
    type: 'GET',
    success: this._makeResponseHandler(this._didRemoveDms, [session]),
    error: this._makeResponseHandler(this._didFailRemoveDms, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didRemoveDms = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(REMOVE_DMS, args);
  this.dispatchEvent(e);
};

prototype._didFailRemoveDms = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, REMOVE_DMS_ERROR);
};

/**
 * Fetches the recordable storage list of the DMS specified.
 *
 * The DMS must support HDRL.
 * (i.e. <em>hdlnkdoc</em> property of the DMS object
 *  must be "REC-1.00")
 *
 * @method recDestList
 * @param {Object} request
 * @param {String} request.udn
 * @return {ContentsTransferApiClientSession}
 */
prototype.recDestList = function(request) {
  var slice = Array.prototype.slice;
  var api = 'rec_dest_list';
  var session;
  var req;
  var query = {};

  if (!!!request || !!!(query.udn = request.udn)) {
    throw new Error(api + ': udn is required');
  };

  session = new ContentsTransferApiClientSession(api);

  req = {
    data: query,
    success: this._makeResponseHandler(this._didFetchRecDestList, [session]),
    error: this._makeResponseHandler(this._didFailFetchRecDestList, [session]),
    cache: false,
    dataType: 'json',
    context: this
  };
  session.query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didFetchRecDestList = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(REC_DEST_LIST, args);
  this.dispatchEvent(e);
};

prototype._didFailFetchRecDestList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, REC_DEST_LIST_ERROR);
};

/**
 * Transfers specified contents.
 *
 * @method transfer
 * @param {Object} request
 * @param {String} request.destUdn Destination device UDN.
 * @param {String} request.destId Destination storage ID of the device.
 * @param {String[]} request.srcIds Object IDs to be transferred.
 * @param {Boolean} [request.shutdown=false] If true, turns off the destination device after the transferring has finished. It is ignored if the device doesn't have the function.
 * @param {Date} [request.startTime] Enables programmed transferring. The transferring will start the date specified.
 * @return {ContentsTransferApiClientSession}
 */
prototype.transfer = function(request) {
  var api = 'transfer';
  var session;
  var req;
  var query = {};

  query.dst_udn = request.destUdn;
  query.src_ids = request.srcIds.join(',');

  if (!!request.destId) {
    query.dst_id = request.destId;
  }

  if (!!request.shutdown) {
    query.shutdown = 1;
  }

  if (!!request.startTime) {
    query.start_time = Formatter.dateToDcDate(request.startTime);
  }

  session = new ContentsTransferApiClientSession(api);

  req = {
    data: query,
    type: 'POST',
    success: this._makeResponseHandler(this._didSucceedTransferAction, [session]),
    error: this._makeResponseHandler(this._didFailTransferAction, [session]),
    dataType: 'json',
    context: this
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didSucceedTransferAction = function(data, textStatus, jqXhr, session) {
  dixim.array.remove(this._sessions, session);
  this.dispatchEvent(new Event(TRANSFER, {session: session}));
};

prototype._didFailTransferAction = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, TRANSFER_ERROR);
};

/**
 * Downloads contents from the server specified.
 *
 * @method dnTransfer
 * @param {Object} request
 * @param {String} request.srcUdn Source device UDN.
 * @param {String[]} request.srcIds Object IDs to be downloaded.
 * @param {String} [request.destId] Container ID of the local DMS where the downloaded files are to be saved.
 * @param {Boolean} [request.shutdown=false] If true, turns off the source device after the transferring has finished. It is ignored if the device doesn't have the function.
 * @param {Date} [request.startTime] Enables programmed download. The download will start the date specified.
 * @return {ContentsTransferApiClientSession}
 */
prototype.dnTransfer = function(request) {
  var api = 'dn_transfer';
  var session;
  var req;
  var query = {};

  query.src_udn = request.srcUdn;
  query.src_ids = request.srcIds.join(',');

  if (typeof request.destId !== 'undefined') {
    query.dst_id = request.destId;
  }

  if (!!request.shutdown) {
    query.shutdown = 1;
  }

  if (!!request.startTime) {
    query.start_time = Formatter.dateToDcDate(request.startTime);
  }

  session = new ContentsTransferApiClientSession(api);

  req = {
    data: query,
    type: 'POST',
    success: this._makeResponseHandler(this._didSucceedDnTransferAction, [session]),
    error: this._makeResponseHandler(this._didFailDnTransferAction, [session]),
    dataType: 'json',
    context: this
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didSucceedDnTransferAction = function(data, textStatus, jqXhr, session) {
  dixim.array.remove(this._sessions, session);
  this.dispatchEvent(new Event(DN_TRANSFER, {session: session}));
};

prototype._didFailDnTransferAction = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, DN_TRANSFER_ERROR);
};

/**
 * Fetches the transfer status.
 *
 * @method transferStatus
 * @return {ContentsTransferApiClientSession}
 */
prototype.transferStatus = function() {
  var api = 'transfer_status';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  if (!!this._pin) {
    query.pin_code = this._pin;
  }

  req = {
    cache: false,
    success: this._makeResponseHandler(this._didFetchTransferStatus, [session]),
    error: this._makeResponseHandler(this._didFailFetchTransferStatus, [session]),
    dataType: 'json',
    data: query,
    context: this
  };
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  session.query = query;
  this._sessions.push(session);

  return session;
};

prototype._didFetchTransferStatus = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(TRANSFER_STATUS, args);
  this.dispatchEvent(e);
};

prototype._didFailFetchTransferStatus = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, TRANSFER_STATUS_ERROR);
};

/**
 * Invokes the cancel_transfer action.
 *
 * @method cancelTransfer
 * @return {ContentsTransferApiClientSession}
 */
prototype.cancelTransfer = function() {
  var api = 'cancel_transfer';
  var session;
  var req;

  session = new ContentsTransferApiClientSession(api);

  req = {
    cache: false,
    success: this._makeResponseHandler(this._didCancelTransfer, [session]),
    error: this._makeResponseHandler(this._didFailCancelTransfer, [session]),
    dataType: 'json',
    context: this
  };
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didCancelTransfer = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(CANCEL_TRANSFER, args);
  this.dispatchEvent(e);
};

prototype._didFailCancelTransfer = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, CANCEL_TRANSFER_ERROR);
};

/**
 * Invokes the rename action.
 *
 * @method rename
 * @param {Object} request
 * @param {String} request.id
 * @param {String} request.newName
 * @return {ContentsTransferApiClientSession}
 */
prototype.rename = function(request) {
  var api = 'rename';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  query.id = request.id;
  query.new_name = request.newName;

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didRename, [session]),
    error: this._makeResponseHandler(this._didFailRename, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didRename = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(RENAME, args);
  this.dispatchEvent(e);
};

prototype._didFailRename = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, RENAME_ERROR);
};

/**
 * get token.
 *
 * @method getTocken
 * @param {Object} request
 * @return {ContentsTransferApiClientSession}
 */
prototype.getToken = function(request) {
  var api = 'getToken';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didGetToken, [session]),
    error: this._makeResponseHandler(this._didFailGetToken, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), "get_token"].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didGetToken = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.data = data;
  args.session = session;
  e = new Event(GET_TOKEN, args);
  this.dispatchEvent(e);
};

prototype._didFailGetToken = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, GET_TOKEN_ERROR);
};

/**
 * Invokes the remove action.
 *
 * @method remove
 * @param {Object} request
 * @param {String} request.id
 * @param {String} request.token
 * @return {ContentsTransferApiClientSession}
 */
prototype.remove = function(request) {
  var api = 'remove';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  query.id = request.id;
  query.token = request.token;

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didRemove, [session]),
    error: this._makeResponseHandler(this._didFailRemove, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didRemove = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(REMOVE, args);
  this.dispatchEvent(e);
};

prototype._didFailRemove = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, REMOVE_ERROR);
};

/**
 * Tests the pin code.
 *
 * @method matchPinCode
 * @param {Object} request
 * @param {String} request.code
 * @return {ContentsTransferApiClientSession}
 */
prototype.matchPinCode = function(request) {
  var api = 'match_pin_code';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  query.code = request.code;

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didMatchPinCode, [session]),
    error: this._makeResponseHandler(this._didFailMatchPinCode, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.query = query;
  session.request = request;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didMatchPinCode = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};
  var request = session.request;

  dixim.array.remove(this._sessions, session);

  if (typeof request.cache === 'undefined' || !!request.cache) {
    this._setPin(request.code);
  }

  args.session = session;
  args.pin = request.code;
  e = new Event(MATCH_PIN_CODE, args);
  this.dispatchEvent(e);
};

prototype._didFailMatchPinCode = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, MATCH_PIN_CODE_ERROR);
};

/**
 * Fetches log list.
 *
 * @method logList
 * @return {ContentsTransferApiClientSession}
 */
prototype.logList = function() {
  var api = 'log_list';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  if (this._pin) {
    query.pin_code = this._pin;
  }

  req = {
    cache: false,
    type: 'GET',
    success: this._makeResponseHandler(this._didFetchLogList, [session]),
    error: this._makeResponseHandler(this._didFailFetchLogList, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didFetchLogList = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(LOG_LIST, args);
  this.dispatchEvent(e);
};

prototype._didFailFetchLogList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, LOG_LIST_ERROR);
};

/**
 * Clears log list.
 *
 * @method clearLogList
 * @return {ContentsTransferApiClientSession}
 */
prototype.clearLogList = function() {
  var api = 'clear_log_list';
  var session;
  var req;

  session = new ContentsTransferApiClientSession(api);

  req = {
    cache: false,
    type: 'GET',
    success: this._makeResponseHandler(this._didClearLogList, [session]),
    error: this._makeResponseHandler(this._didFailClearLogList, [session]),
    dataType: 'json',
    context: this
  };

  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didClearLogList = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(CLEAR_LOG_LIST, args);
  this.dispatchEvent(e);
};

prototype._didFailClearLogList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, CLEAR_LOG_LIST_ERROR);
};

/**
 * Fetches reservation list.
 *
 * @method reservationList
 * @return {ContentsTransferApiClientSession}
 */
prototype.reservationList = function() {
  var api = 'reservation_list';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  if (this._pin) {
    query.pin_code = this._pin;
  }

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didFetchReservationList, [session]),
    error: this._makeResponseHandler(this._didFailFetchReservationList, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didFetchReservationList = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(RESERVATION_LIST, args);
  this.dispatchEvent(e);
};

prototype._didFailFetchReservationList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, RESERVATION_LIST_ERROR);
};

/**
 * Updates reservation.
 *
 * @method updateReservation
 * @param {Object} request
 * @param {String} request.id
 * @param {Date} request.startTime
 * @return {ContentsTransferApiClientSession}
 */
prototype.updateReservation = function(request) {
  var api = 'update_reservation';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  query.id = request.id;
  query.start_time = Formatter.dateToDcDate(request.startTime);

  req = {
    cache: false,
    type: 'GET',
    success: this._makeResponseHandler(this._didUpdateReservation, [session]),
    error: this._makeResponseHandler(this._didFailUpdateReservation, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didUpdateReservation = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(UPDATE_RESERVATION, args);
  this.dispatchEvent(e);
};

prototype._didFailUpdateReservation = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, UPDATE_RESERVATION_ERROR);
};

/**
 * Removes reservation.
 *
 * @method removeReservation
 * @param {Object} request
 * @param {String} request.id
 * @return {ContentsTransferApiClientSession}
 */
prototype.removeReservation = function(request) {
  var api = 'remove_reservation';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  query.id = request.id;

  req = {
    cache: false,
    type: 'GET',
    success: this._makeResponseHandler(this._didRemoveReservation, [session]),
    error: this._makeResponseHandler(this._didFailRemoveReservation, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didRemoveReservation = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(REMOVE_RESERVATION, args);
  this.dispatchEvent(e);
};

prototype._didFailRemoveReservation = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, REMOVE_RESERVATION_ERROR);
};

/**
 * Invokes get_requested_transfer_list.
 *
 * @method getRequestedTransferList
 * @return {ContentsTransferApiClientSession}
 */
prototype.getRequestedTransferList = function() {
  var api = 'get_requested_transfer_list';
  var session;
  var req;
  var query = {};

  session = new ContentsTransferApiClientSession(api);

  if (this._pin) {
    query.pin_code = this._pin;
  }

  req = {
    cache: false,
    type: 'GET',
    success: this._makeResponseHandler(this._didFetchRequestedTransferList, [session]),
    error: this._makeResponseHandler(this._didFailFetchRequestedTransferList, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.query = query;
  session.connection = $.ajax([Configs.get('api'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didFetchRequestedTransferList = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(GET_REQUESTED_TRANSFER_LIST, args);
  this.dispatchEvent(e);
};

prototype._didFailFetchRequestedTransferList = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, GET_REQUESTED_TRANSFER_LIST_ERROR);
};

/**
 * Get the permission information of the compressed storage.
 *
 * @method fetchAvailabilityCompression
 * @return {ContentsTransferApiClientSession}
 */
prototype.fetchAvailabilityCompression = function(request) {
  var session;
  var cfg;

  session = new ContentsTransferApiClientSession('fetch_availability');

  cfg = {
    type: 'POST',
    dataType: 'json',
    success: this._makeResponseHandler(this._didFetchAvailability, [session]),
    error: this._makeResponseHandler(this._didFailFetchAvailability, [session]),
    context: this
  };
  session.request = request;
  session.connection = $.ajax(Configs.get('api_autocompress') + '/availability/get', cfg);
  this._sessions.push(session);

  return session;
};

prototype._didFetchAvailability = function(data, textStatus, jqXhr, session) {
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.data = data;

    event = new Event(FETCH_AVAILABILITY_COMPRESSION, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailFetchAvailability = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, FETCH_AVAILABILITY_COMPRESSION_ERROR);
};
/**
 * Set the permission information of the compressed storage.
 *
 * @method updateAvailabilityCompression
 * @param {Object} request
 * @param {Boolean} request.availability
 * @return {ContentsTransferApiClientSession}
 */
prototype.updateAvailabilityCompression = function(request) {
  var query;
  var session;
  var cfg;

  session = new ContentsTransferApiClientSession('update_availability');

  query = {availability: request.availability};
  cfg = {
    type: 'POST',
    dataType: 'json',
    success: this._makeResponseHandler(this._didUpdateAvailability, [session]),
    error: this._makeResponseHandler(this._didFailUpdateAvailability, [session]),
    context: this,
    data: query
  };
  session.request = request;
  session.connection = $.ajax(Configs.get('api_autocompress') + '/availability/set', cfg);
  this._sessions.push(session);

  return session;
};

prototype._didUpdateAvailability = function(data, textStatus, jqXhr, session) {
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.data = data;

    event = new Event(UPDATE_AVAILABILITY, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailUpdateAvailability = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, UPDATE_AVAILABILITY_ERROR);
};

/**
 * Get a preset that is set.
 *
 * @method fetchPreset
 * @return {ContentsTransferApiClientSession}
 */
prototype.fetchPreset = function(request) {
  var session;
  var cfg;

  session = new ContentsTransferApiClientSession('fetch_preset');

  cfg = {
    type: 'POST',
    dataType: 'json',
    success: this._makeResponseHandler(this._didFetchPreset, [session]),
    error: this._makeResponseHandler(this._didFailFetchPreset, [session]),
    context: this
  };
  session.request = request;
  session.connection = $.ajax(Configs.get('api_autocompress') + '/preset/get', cfg);
  this._sessions.push(session);

  return session;
};

prototype._didFetchPreset = function(data, textStatus, jqXhr, session) {
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.data = data;

    event = new Event(FETCH_PRESET, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailFetchPreset = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, FETCH_PRESET_ERROR);
};
/**
 * Want to select a preset.
 *
 * @method updatePreset
 * @param {Object} request
 * @param {Number} request.id
 * @return {ContentsTransferApiClientSession}
 */
prototype.updatePreset = function(request) {
  var query;
  var session;
  var cfg;

  session = new ContentsTransferApiClientSession('update_preset');

  query = {id: request.id};
  cfg = {
    type: 'POST',
    dataType: 'json',
    success: this._makeResponseHandler(this._didUpdatePreset, [session]),
    error: this._makeResponseHandler(this._didFailUpdatePreset, [session]),
    context: this,
    data: query
  };
  session.request = request;
  session.connection = $.ajax(Configs.get('api_autocompress') + '/preset/set', cfg);
  this._sessions.push(session);

  return session;
};

prototype._didUpdatePreset = function(data, textStatus, jqXhr, session) {
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.data = data;

    event = new Event(UPDATE_PRESET, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailUpdatePreset = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, UPDATE_PRESET_ERROR);
};
/**
 * Will register the specified item in compression reservation queue..
 *
 * @method addReservationCompression
 * @param {Object} request
 * @param {Number} request.objectId
 * @return {ContentsTransferApiClientSession}
 */
prototype.addReservationCompression = function(request) {
  var query;
  var session;
  var cfg;

  session = new ContentsTransferApiClientSession('add_reservation_compression');

  query = request;
  cfg = {
    type: 'POST',
    dataType: 'json',
    success: this._makeResponseHandler(this._didReservationCompression, [session]),
    error: this._makeResponseHandler(this._didFailReservationCompression, [session]),
    context: this,
    data: query
  };
  session.request = request;
  session.connection = $.ajax(Configs.get('api_autocompress') + '/reservation/add', cfg);
  this._sessions.push(session);

  return session;
};

prototype._didReservationCompression = function(data, textStatus, jqXhr, session) {
  var args = {};
  var event;

  dixim.array.remove(this._sessions, session);

  if (!session.isCancelled()) {
    args.session = session;
    args.data = data;

    event = new Event(RESERVATION_COMPRESSION, args);
    this.dispatchEvent(event);
  }
};

prototype._didFailReservationCompression = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, RESERVATION_COMPRESSION_ERROR);
};
}());
