/**
 * @module ctt
 * @submodule ticker-view
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var $ = dixim.ctt.jQuery;

var TickerView;
var prototype;

/**
 * Display short message box.
 *
 * @class TickerView
 * @constructor
 * @param {Object} configs
 */
TickerView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(TickerView, EventTarget);
dixim.ctt.TickerView = TickerView;
prototype = TickerView.prototype;

/**
 * @event render
 */
var RENDER = 'render';

/**
 * Triggered when the ticker has been hidden.
 *
 * @event hide
 */
var HIDE = 'hide';

/**
 * Triggered when the ticker has been shown.
 *
 * @event show
 */
var SHOW = 'show';

var TEMPLATE_TICKER = '<div class="ctt-ticker ${type}" style="display: none;">${text}</div>';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 */
prototype._initialize = function(configs) {
  this._text = configs.text || '';
  this._type = configs.type || 'normal';
  this._containerNode = configs.containerNode || document;
};

/**
 * @method _createView
 * @private
 */
prototype._createView = function() {
  var template;

  template = TEMPLATE_TICKER.
             replace('${text}', this._text).
             replace('${type}', this._type === 'error' ? 'ctt-ticker-error' : '');
  this._view = $(template);

  this.dispatchEvent(new Event(RENDER));
};

/**
 * @method show
 * @param {boolean} [animated=true]
 */
prototype.show = function(animated) {
  this._show(animated);
};

prototype._show = function(animated) {
  if (!!!this._view) {
    this._createView();
  }

  // TODO: hover
  if (typeof animated === 'undefined' || animated) {
    this._view.appendTo($(this._containerNode)).slideDown(100, $.proxy(this._didShow, this));
  } else {
    this._view.appendTo($(this._containerNode)).show();
    this._didShow();
  }
};

prototype._didShow = function() {
  this.dispatchEvent(new Event(SHOW));
};

/**
 * @method hide
 * @param {boolean} [animated=true]
 */
prototype.hide = function(animated) {
  this._hide(animated);
};

prototype._hide = function(animated) {
  if (!!!this._view) return;

  if (typeof animated === 'undefined' || animated) {
    this._view.slideUp(100, $.proxy(function() {
      this._view.detach();
      this._didHide();
    }, this));
  } else {
    this._view.detach();
    this._didHide();
  }
};

prototype._didHide = function() {
  this.dispatchEvent(new Event(HIDE));
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  if (!!!this._view) return;
  this._view.hide();
  this._view = null;
};

/**
 * @method setText
 * @param {String} text
 */
prototype.setText = function(text) {
  this._setText(text);
};

prototype._setText = function(text) {
  this._text = text;
  this._didChangeText();
};

prototype._didChangeText = function() {
  if (!!this._view) {
    this._view.html(this._text);
  }
};

///**
// * @method setParent
// * @param {String|jQuery} parent parent node selector string or jQuery object.
// */
//prototype.setParent = function(parent) {
//  if (!!parent) {
//    this._view.appendTo($(parent));
//  } else {
//    this._view.detach();
//  }
//};

/**
 * @method setType
 * @param {String} type "normal"(default) | "error"
 */
prototype.setType = function(type) {
  this._type = type;
  this._didChangeType();
};

prototype._didChangeType = function() {
  if (!!!this._view) return;

  if (this._type === 'error') {
    this._view.addClass('ctt-ticker-error');
  } else {
    this._view.removeClass('ctt-ticker-error');
  }
};

}());
