/**
 * @module ctt
 * @submodule unlocking-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var UnlockingView = dixim.ctt.UnlockingView;
var AutoDeleteApiClient = dixim.ctt.api.AutoDeleteApiClient;
var _ = dixim.ctt.ResourceBundle.get;

var UnlockingController;
var prototype;

/**
 * @class UnlockingController
 * @constructor
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
UnlockingController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(UnlockingController, EventTarget);
dixim.ctt.UnlockingController = UnlockingController;
prototype = UnlockingController.prototype;

/**
 * Triggered when the unlocking has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.unlocked true if the unlocking is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = configs.model || new AutoDeleteApiClient();
  this._targetItems = configs.targetItems;
  this._view = new UnlockingView({targetItems: this._targetItems});
  this._session = null;
  this._queue = [];
  this._unlocked = false;
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var m = this._model;

  v.addEventListener('unlock', this._didClickUnlock, this);
  v.addEventListener('cancel', this._didClickCancel, this);

  m.addEventListener('unlock', this._didActionUnlock, this);
  m.addEventListener('unlockError', this._didFailActionUnlock, this);

  v.show();
};

prototype._didClickUnlock = function(e) {
  if (this._queue.length) return;

  this._view.showActivityIndicator();

  this._queue = this._targetItems.concat();
  this._unlocked = true;
  this._processQueue();
};

prototype._processQueue = function() {
  var item;
  var total = this._targetItems.length;
  var queue = this._queue;

  if (!queue.length) {
    this._didFinishUnlocking();
    return;
  }

  item = queue.shift();
  this._view.updateProgress(item, total - queue.length - 1, total);
  this._session = this._model.unlock({objectIds: item.id, protectFlag: false});
};

prototype._didActionUnlock = function(e) {
  this._session = null;
  this._processQueue();
};

prototype._didFailActionUnlock = function(e) {
  this._view.hideActivityIndicator();
  this._session = null;
  this._queue = [];
  this._view.showError(_('ctt_unlocking_view_error'));
};

prototype._didClickCancel = function(e) {
  if (!this._queue.length && !!!this._session) {
    this._view.destroy();
    this.dispatchEvent(new Event(CLOSE, {unlocked: this._unlocked}));
    return;
  }
  this._queue = [];
};

prototype._didFinishUnlocking = function(e) {
  this._view.hideActivityIndicator();
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE, {unlocked: this._unlocked}));
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

}());
